;-----------------------------------------------------------------------------------------
; Titel			:	DOGM162W-A
;				:
; Functions		:	LCD_line1:
; 				:	LCD-Output of a Voltage in 4 Measuring Ranges, each with 10 Bits
;				:	  0.00000 until   0.25550 {V]	Resolution: 0.00025 [V] DC or AC
;				:	  0.0000  until   2.5550  (V)	Resolution: 0.0025  [V] DC or AC
;				:	  0.000   until  25.550	  (V)	Resolution: 0.025   [V] DC or AC
;				:	  0.00    until 255.50	  (V)	Resolution: 0.25    [V] DC or AC
;				:
;				:	LCD_line2:
;				:	To be selected with the white push button:
;				:	 BAR GRAPH TYPE 0			Empty									
;				;	 BAR GRAPH TYPE 1			"Quasianalogue" with 0 until 80 Bars
;				;	 BAR GRAPH TYPE 2			"Quasianalogue" with 0 until 48 Bars
;				:	 CALIBRATION MODE			" CALIBR. R24=nnn", nnn = 000 until 080
;				:	START STOP for MAX-MIN function
;				:	
;				:	LCD_line2:
;				:	To be selected with the green push button:
;				:	 DC Range: AUTO		automatically after "POWER ON"
;				:	 DC Range:   0.25 V
;				:	 DC Range:   2.5 V
;				:	 DC Range:  25 V
;				:	 DC Range: 250 V
;				:	 AC Range: AUTO 
;				:	 AC Range:   0.25 V
;				:	 AC Range:   2.5 V
;				:	 AC Range:  25 V
;				:	 AC Range: 250 V
;				;	 DC Range: AUTO	MAX MIN	| Start Stop with the white pushbutton 
;				:	
; Diagrams		:	Entire Diagram
; 				:	Upper Circuit Board
; 				:	Lower Circuit Board
;				;	CGRam Type 1
;				;	CGRam Type 2
;				:
;-----------------------------------------------------------------------------------------
; Register		:	No		used		local		| Function 1		| Function 2
;				:			Y	N		Y	N
;				:
;				:	R0		X			X			| Multiplication	| Calculation a
;				:	R1		X				X		| of 2 Values,		| Decimal Number
;				:	R2		X				X		| each 16 Bit		| from a 16 Bit
;				:	R3		X				X		|					| Binary Number
;				:	R4		X				X		|					|
;				:	R5		X				X		|					|
;				:	R6		X				X		|					|
;
;				:	R7		X				X		| R7 =1: DC Auto	 
;				:									| R7 =2: DC   0,25 V
;				:									| R7 =3: DC   2.5  V
;				:									| R7 =4: DC  25    V
;				:									| R7 =5: DC 250    V
;				:									| R7 =6: AC Auto
;				:									| R7 =7: AC   0.25 V
;				:									| R7 =8: AC   2.5  V
;				:									| R7 =9: AC  25    V
;				:									| R7 =10:AC 250    V
;				:									| R7 =11:MAX MIN DC Auto
;
;				:	R8		X				X		| R8 =1  MR 250    V
;				:									| R8 =2  MR  25    V
;				:									| R8 =4  MR   2.5  V
;				:									| R8 =16 MR   0.25 V
;
;				:	R9		X				X		| R9 =0  MAX MIN Off
;				:									| R9 =1  MAX MIN On
;
;				:	R10		X				X		| R10=1: Battery Low
;				:									| R10=2: Continuity Test
;				:									| R10=3: Overflow
;				:									| R10=4: Calibration Mode
;
;				:	R11		X				X		| MIN V DC Low
;				:	R12		X				X		| MIN V DC High
;				:	R13		X				X		| MAX V DC Low
;				:	R14		X				X		| MAX V DC High
;
;				:	R15			X
;				:	R16		X				X		| diverse
;				:	R17		X			X			| diverse			| ADC Value Correction
;				:	R18		X			X			| diverse			| ADC Value Correction
;				:	R19		X			X			| diverse			| Fields in LCD_line2
;				:	R20		X			X								| Lines  in LCD_line2
;				:	R21		X			X								| Blanks in LCD_line2
;				:	R22		X				X		| ADC Low
;				:	R23		X				X		| ADC High
;				:	R24		X				X		| 0...80 Bars in LCD_line2
;				:	R25		X			X			| ADC Value Correction Low
;				:	R26		X			X			| ADC Value Correction High
;
;				:	R27		X				X		| R27=0: Bar Graph None
;				:									| R27=1: Bar Graph Type 1
;				:									| R27=2: Bar Graph Type 2
;				:									| R27=3: Calibration Mode
;
;				:	R28			X
;				:	R29			X
;				:	R30		X				X		| ZL
;				:	R31		X				X		| ZH
;-----------------------------------------------------------------------------------------
; Processor		:	ATmega8A-PU
; Takt (Quarz)	:	4.096 MHz
; Language		:	Assembler Studio 4
; Date			:	31.10.2014
; Version		:	1.0
; Autor			:	KlausD 
;-----------------------------------------------------------------------------------------
 		.include 	"m8def.inc"
;-----------------------------------------------------------------------------------------
; Reset and Interrupt Vector				Description

A000:	rjmp	Main						;  1 POWER ON RESET
		reti								;  2 Int0-Interrupt
		reti								;  3 Int1-Interrupt
		reti								;  4 TC2 Compare Match
		reti								;  5 TC2 Overflow
		reti								;  6 TC1 Capture
		reti								;  7 TC1 Compare Match A
		reti								;  8 TC1 Compare Match B
		reti								;  9 TC1 Overflow
		rjmp	onTMR0						; 10 TC0 Overflow
		reti								; 11 SPI, STC Serial Transfer Complete
		reti								; 12 UART Rx complete
		reti								; 13 UART Data Register Empty
		reti								; 14 UART Tx complete
		rjmp	onADC						; 15 ADC Conversion Complete
		reti								; 16 EEPROM Ready
		reti								; 17 Analog Comparator
		reti								; 18 TWI (IC) Serial Interface
		reti								; 19 Store Program Memory Redy

;-----------------------------------------------------------------------------------------
; Start, Power ON, Reset, Ports, Interrupt

Main:
		ldi		R16		,	LOW (RAMEND)	; for  Stackpointer LOW
		out		SPL		,	R16				; INIT Stackpointer LOW

		ldi		R16		,	HIGH(RAMEND)	; for  Stackpointer HIGH
		out		SPH		,	R16				; INIT Stackpointer HIGH

		ldi		R16		,	0b00111111		; PORTB Bits 0 until 5 = Output
		out		DDRB	,	R16				; PORTB Bits 6 and   7 = Quartz
		ldi		R16		,	0b00000000		; PORTB Bits 0 until 7 = 0
		out		PORTB	,	R16				; 

		ldi		R16		,	0b00000000		; PORTC Bits 0 until 7 = Input
		out		DDRC	,	R16				; 
		ldi		R16		,	0b00001111		; PORTC Bits 0 until 3 = 1 (Pullup)
		out		PORTC	,	R16				; PORTC Bit  4 = ADC Input for DC
											; PORTC Bit  5 = ADC Input for AC
											; Pullup would adulerate the Measuring Value !

		ldi		R16		,	0b11111111		; PORTD Bits 0 until 7 = Output
		out		DDRD	,	R16				;
		ldi		R16		,	0b00000000		; PORTD Bits 0 until 7 = 0
		out		PORTD	,	R16				;

;-----------------------------------------------------------------------------------------
; Timer/Counter2: 64 kHz an B.3

		ldi		R16		,	(1<<COM20) | (1<<WGM21) | (1<<CS20)	
		out		TCCR2	,	R16				; COM20 = Toggle B.3
											; WGM21 = CTC, TOP OCR2
											; CS20  = Prescaling 1

		ldi		R16		,	0b00011111		; 4096000 / 32 = 128000 Hz (B.3 = 64000 Hz)
		out		OCR2	,	R16

		ldi		R16		,	(1<<OCIE2)		; Timer/counter2 Output Compare 
		out		TIMSK	,	R16				; Match Interrupt Enable

;-----------------------------------------------------------------------------------------
; ADC

		ldi		R16		,	(1<<ADEN) | (1<<ADIF) | (1<<ADIE) | (1<<ADPS2) | (1<<ADPS0)
		out		ADCSRA	,	R16				; ADC Enable
											; ADC Interrupt Flag
											; ADC Interrupt Enable
											; ADC Prescale 32

;-----------------------------------------------------------------------------------------
; 8-Bit Timer/Counter0

		ldi		R16		,	(1<<CS02) | (1<<CS00)
		out		TCCR0	,	R16				; Prescale 1024

		ldi		R16		,	131				; count upwords 131 bis 256 = Overflow
		out		TCNT0	,	R16				; these are 125 Increments

		ldi		R16		,	(1<<OCIE2) | (1<<TOIE0)
		out		TIMSK	,	R16				; OCIE2 = Timer/Counter2 Output Compare
											; 		  Match Interrupt Enable
											; TOIE0 = Timer/Counter0 Overflow Interrupt
											; 		  Enable

;-----------------------------------------------------------------------------------------

		ldi		R16		,	0b00010000
		mov		R8		,	R16
		out		PORTB	,	R16				; Swith on Measuring Range 0.25 V
											; B.0 =  1 = 0b00000001: MR 250.00    V
											; B.1 =  2 = 0b00000010: MR  25.000   V
											; B.2 =  4 = 0b00000100: MR   2.5000  V
											; B.4 = 16 = 0b00010000: MR   0.25000 V
											; B.3 = 64 KHz for the charge pump
											; 		   whitch creates - 5 VDC
											; B.5 = 1/2 Scanning Frequency

		ldi		R16		,	1				; MR = DC AUTO	(selection with Push Button)
		mov		R7		,	R16
		ldi		R27		,	2				; R27 = 0: Bar Graph None
											; R27 = 1: Bar Graph Type 1 (all bars)
											; R27 = 2: Bar Graph Type 2 (every second bar)
											; R27 = 3: Calibration Mode
		rcall	LCD_init
		rcall	LCD_clear
		rjmp	A025						; First check of the Measuring Range

;-----------------------------------------------------------------------------------------
; Main Loop

Loop:
		sei									; Allow Interrupt
		rcall	CG00						; LCD_line2 Bar Graph anzeigen

		ldi		R16		,	0				;
		mov		R10		,	R16				;

		ldi		R16		,	5
A001:
		sbic	PINC	,	3				; jump over /Continuity Test
		rjmp	A004						; Eine offene Leitung kann Strungen
		dec		R16							; einfangen. Der Lautsprecher knackt dann.
		tst		R16							; Alternative zur dieser Mehrfachabtastung:
		brne	A001						; Ein Pullup-Widerstand von z.B. 10 kOhm.

;-----------------------------------------------------------------------------------------
; /Continuity Test

A002:										;
		ldi		ZL		,	LOW(Text04_ContTest*2)
		ldi		ZH		,	HIGH(Text04_ContTest*2)
		rcall	H201						;
A003:										;
		sbi		PORTD	,	7				; ca. 1 kHz Ton
		rcall	wait05ms					;
		cbi		PORTD	,	7				;
		rcall	wait05ms					;
		sbis	PINC	,	3				;
		rjmp	A003																;
		ldi		R16		,	2				;
		mov		R10		,	R16				;
		rjmp	Loop						;

;-----------------------------------------------------------------------------------------
; /Battery Low
											;
A004:
		ldi		R16		,	0				;
		mov		R10		,	R16				;
		sbic	PINC	,	1				;
		rjmp	A005						;
		ldi		R16		,	1				;
		mov		R10		,	R16				; R10 = 1: Battery Low
A005:
		mov		R16		,	R7				; Selection of the ADC Channel
		cpi		R16		,	1				; R7 = 1, 2, 3, 4,  5 Channel 5 Pin 28
		breq	A006						; R7 = 6, 7, 8, 9, 10 Channel 4 Pin 27
		cpi		R16		,	2
		breq	A006						;  R7
		cpi		R16		,	3				; 
		breq	A006						;   1	DC AUTO 
		cpi		R16		,	4				;	2	DC 0.25 V
		breq	A006						;	3	DC 2.50 V
		cpi		R16		,	5				;	4	DC 25.0 V
		breq	A006						;	5	DC 250  V
		cpi		R16		,	6				;
		breq	A007						;	6	AC AUTO 
		cpi		R16		,	7				;	7	AC 0.25 V
		breq	A007						;	8	AC 2.50 V
		cpi		R16		,	8				;	9	AC 25.0 V
		breq	A007						;  10	AC 250  V
		cpi		R16		,	9
		breq	A007
		cpi		R16		,	10
		breq	A007
A006:										; ADC Channel C.4 Pin 27
		ldi		R16		,	(1<<MUX2)
		out		ADMUX	,	R16
		rjmp	A008
A007:										; ADC Channel C.5 Pin 28
		ldi		R16		,	(1<<MUX2) | (1<<MUX0)
		out		ADMUX	,	R16
		rjmp	A008
A008:
		sbi		ADCSRA	,	ADSC			; next ADC action
A009:
		sbic	ADCSRA	,	ADSC			; wait until ADC action ready
		rjmp	A009
		rcall	onADC

;-----------------------------------------------------------------------------------------
; Determing Measuring Range in AUTO Mode

		push	R16
		mov		R16		,	R7
		cpi		R16		,	1				; Range = DC AUTO 
		breq	A010
		cpi		R16		,	2				; Range = DC 0.25 V
		breq	A011
		cpi		R16		,	3				; Range = DC 2.55 V
		breq	A012
		cpi		R16		,	4				; Range = DC 25.5 V
		breq	A013
		cpi		R16		,	5				; Range = DC 255  V
		breq	A014
		cpi		R16		,	6				; Range = AC AUTO 
		breq	A015
		cpi		R16		,	7				; Range = AC 0.25 V
		breq	A016
		cpi		R16		,	8				; Range = AC 2.55 V
		breq	A017
		cpi		R16		,	9				; Range = AC 25.5 V
		breq	A018
		cpi		R16		,	10				; Range = AC 255  V
		breq	A019
A010:
		rjmp	A020						; MR AUTO 
A011:
		ldi		R16		,	16				; MR 0.25 V
		mov		R8		,	R16
		out		PORTB	,	R16
		pop		R16
		rjmp	A040
A012:
		ldi		R16		,	4				; MR 2.55 V
		mov		R8		,	R16
		out		PORTB	,	R16
		pop		R16
		rjmp	A040
A013:
		ldi		R16		,	2				; MR 25.5 V
		mov		R8		,	R16
		out		PORTB	,	R16
		pop		R16
		rjmp	A040
A014:
		ldi		R16		,	1				; MR 255 V
		mov		R8		,	R16
		out		PORTB	,	R16
		pop		R16
		rjmp	A040
A015:
		rjmp	A020						; MR AUTO
A016:
		ldi		R16		,	16				; MR 0.25 V
		mov		R8		,	R16
		out		PORTB	,	R16
		pop		R16
		rjmp	A040
A017:
		ldi		R16		,	4				; MR 2.55 V
		mov		R8		,	R16
		out		PORTB	,	R16
		pop		R16
		rjmp	A040
A018:
		ldi		R16		,	2				; MR 25.5 V
		mov		R8		,	R16
		out		PORTB	,	R16
		pop		R16
		rjmp	A040
A019:
		ldi		R16		,	1				; MR 255 V
		mov		R8		,	R16
		out		PORTB	,	R16
		pop		R16
		rjmp	A040
A020:
		mov		R16		,	R8				;
		cpi		R16		,	1				;  MR 250 V already on
		breq	A022
		cpi		R23		,	0b00000011		; R23 = ADCH
		breq	A021
		rjmp	A022
A021:
		cpi		R22		,	0b11111111		; R22 = ADCL
		breq	A025
		rjmp	A022
A022:
		mov		R16		,	R8				;
		cpi		R16		,	16				; MR 0.25 V already on
		breq	A024					;
		cpi		R23		,	0b00000000
		breq	A023
		rjmp	A024
A023:
		cpi		R22		,	0b01000000
		brlo	A025
		rjmp	A024
A024:
		pop		R16
		rjmp	A040

; Maximum in MR 0.25 V ?
A025:
		ldi		R16		,	16				; MR 0.25 V
		mov		R8		,	R16
		out		PORTB	,	R16
		ldi		R16		,	30
		rcall	Wait
		sbi		ADCSRA	,	ADSC			; next ADC conversion
A026:
		sbic	ADCSRA	,	ADSC			; wait until done
		rjmp	A026
A027:
		cpi		R23		,	0b00000011
		breq	A028
		rjmp	A039
A028:
		cpi		R22		,	0b11111111
		breq	A029
		rjmp	A039

; Maximum in MR 2.5 V ?
A029:
		ldi		R16		,	4				; MR 2.5 V
		mov		R8		,	R16
		out		PORTB	,	R16
		ldi		R16		,	30
		rcall	Wait
		sbi		ADCSRA	,	ADSC			; next ADC conversion
A030:
		sbic	ADCSRA	,	ADSC			; wait until done
		rjmp	A030
A031:
		cpi		R23		,	0b00000011
		breq	A032
		rjmp	A039
A032:
		cpi		R22		,	0b11111111
		breq	A033
		rjmp	A039

; Maximum in MR 25 V ?
A033:
		ldi		R16		,	2				; MR 25 V
		mov		R8		,	R16
		out		PORTB	,	R16
		ldi		R16		,	30
		rcall	Wait
		sbi		ADCSRA	,	ADSC			; next ADC conversion
A034:
		sbic	ADCSRA	,	ADSC			; wait until done
		rjmp	A034
A035:
		cpi		R23		,	0b00000011
		breq	A036
		rjmp	A039
A036:
		cpi		R22		,	0b11111111
		breq	A037
		rjmp	A039

; switch on MR 250 V because the lower MRs show maximum
A037:
		ldi		R16		,	1				; MR 250 V
		mov		R8		,	R16
		out		PORTB	,	R16
		ldi		R16		,	30
		rcall	Wait
		sbi		ADCSRA	,	ADSC			; next ADC conversion
A038:
		sbic	ADCSRA	,	ADSC			; wait until done
		rjmp	A038

; finish the search for the Maximum
A039:
		rjmp	Loop

;-----------------------------------------------------------------------------------------
; 1/2 Scanning frequency of the analogue signal

A040:
		mov		R16		,	R1				; TP17: 4,18 Hz
		cpi		R16		,	0
		breq	A041						; Scanning Frequqncy of the Analogue Signal:
		cpi		R16		,	1				; 8,36 Hz
		breq	A042
A041:
		ldi		R16		,	1
		mov		R1		,	R16
		sbi		PORTB	,	5
		rjmp	A043
A042:
		ldi		R16		,	0
		mov		R1		,	R16
		cbi		PORTB	,	5
		rjmp	A043

;-----------------------------------------------------------------------------------------
; R24 - Code

A043:										; R22 sort out into the steps 1 until 20
		cpi		R22		,	244				; R22 from onADC
		brsh	A044
		cpi		R22		,	231
		brsh	A045
		cpi		R22		,	218
		brsh	A046
		cpi		R22		,	205
		brsh	A047
		cpi		R22		,	192
		brsh	A048
		cpi		R22		,	180
		brsh	A049
		cpi		R22		,	167
		brsh	A050
		cpi		R22		,	154
		brsh	A051
		cpi		R22		,	141
		brsh	A052
		cpi		R22		,	128
		brsh	A053
		cpi		R22		,	116
		brsh	A054
		cpi		R22		,	103
		brsh	A055
		cpi		R22		,	90
		brsh	A056
		cpi		R22		,	77
		brsh	A057
		cpi		R22		,	64
		brsh	A058
		cpi		R22		,	52
		brsh	A059
		cpi		R22		,	39
		brsh	A060
		cpi		R22		,	26
		brsh	A061
		cpi		R22		,	13
		brsh	A062
		cpi		R22		,	0
		brsh	A063
A044:
		ldi		R24		,	20
		rjmp	A064
A045:
		ldi		R24		,	19
		rjmp	A064
A046:
		ldi		R24		,	18
		rjmp	A064
A047:
		ldi		R24		,	17
		rjmp	A064
A048:
		ldi		R24		,	16
		rjmp	A064
A049:
		ldi		R24		,	15
		rjmp	A064
A050:
		ldi		R24		,	14
		rjmp	A064
A051:
		ldi		R24		,	13
		rjmp	A064
A052:
		ldi		R24		,	12
		rjmp	A064
A053:
		ldi		R24		,	11
		rjmp	A064
A054:
		ldi		R24		,	10
		rjmp	A064
A055:
		ldi		R24		,	9
		rjmp	A064
A056:
		ldi		R24		,	8
		rjmp	A064
A057:
		ldi		R24		,	7
		rjmp	A064
A058:
		ldi		R24		,	6
		rjmp	A064
A059:
		ldi		R24		,	5
		rjmp	A064
A060:
		ldi		R24		,	4
		rjmp	A064
A061:
		ldi		R24		,	3
		rjmp	A064
A062:
		ldi		R24		,	2
		rjmp	A064
A063:
		ldi		R24		,	1
		rjmp	A064

A064:										; R24 = 2 * R23 + R24
		push	R16							; is being popped in A065, A066, A067 and A068
		cpi		R23		,	0				; R23 from onADC
		breq	A065
		cpi		R23		,	1
		breq	A066
		cpi		R23		,	2
		breq	A067
		cpi		R23		,	3
		breq	A068
A065:
		ldi		R16		,	0
		add		R24		,	R16
		pop		R16
		rjmp	A069
A066:
		ldi		R16		,	20
		add		R24		,	R16
		pop		R16
		rjmp	A069
A067:
		ldi		R16		,	40
		add		R24		,	R16
		pop		R16
		rjmp	A069
A068:
		ldi		R16		,	60
		add		R24		,	R16
		pop		R16
		rjmp	A069

;-----------------------------------------------------------------------------------------
; LCD Line 1: Voltage in [V]

A069:
		push	R17							; will be popped in A085
		ldi		R17		,	48				; converts R2 until R6 into ASCII signs
		mov		R16		,	R8
		cpi		R16		,	16
		breq	A070
		cpi		R16		,	4
		breq	A071
		cpi		R16		,	2
		breq	A072
		cpi		R16		,	1
		breq	A073
A070:
		rjmp	A074						; MR   0.25000 V
A071:
		rjmp	A075						; MR   2.5000  V
A072:
		rjmp	A076						; MR   25.000  V
A073:
		rjmp	A080						; MR   250.00  V
A074:										; MR   0.25000 VDC
		rcall	LCD_line1
		ldi		R16		,	'U'
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'='
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'0'
		rcall	LCD_data
		ldi		R16		,	'.'
		rcall	LCD_data
		mov		R16		,	R6
		add		R16		,	R17
		rcall	LCD_data
		mov		R16		,	R5
		add		R16		,	R17
		rcall	LCD_data
		mov		R16		,	R4
		add		R16		,	R17
		rcall	LCD_data
		mov		R16		,	R3
		add		R16		,	R17
		rcall	LCD_data
		mov		R16		,	R2
		add		R16		,	R17
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'V'
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		rjmp	A085
A075:										; MR   2.5000  VDC
		rcall	LCD_line1
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'U'
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'='
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		mov		R16		,	R6				; R2, R3, R4, R5 and R6:
		add		R16		,	R17				; 1. C001, Multiplication of the ADC - value with 25
		rcall	LCD_data					; 2. D001, Change into decimal numbers 
		ldi		R16		,	'.'				;
		rcall	LCD_data					; 1.:
		mov		R16		,	R5				; the ADC deliveres 10-Bit values from 0 until 1023
		add		R16		,	R17
		rcall	LCD_data
		mov		R16		,	R4
		add		R16		,	R17
		rcall	LCD_data					; 2.:
		mov		R16		,	R3				; the LCD shows ASCII signs
		add		R16		,	R17				; the ASCII signs are being created by the addition
		rcall	LCD_data					; of the Dezimalnumbers plus 48
		mov		R16		,	R2
		add		R16		,	R17				; Example: ASCII '5' = 5 + 48
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'V'
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		rjmp	A085
A076:										; MR  25.000  VDC
		rcall	LCD_line1
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'U'
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'='
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		mov		R16		,	R6				; fade out the leading 0
		cpi		R16		,	0
		breq	A077
		rjmp	A078
A077:
		ldi		R16		,	' '
		rcall	LCD_data
		rjmp	A079
A078:
		mov		R16		,	R6
		add		R16		,	R17
		rcall	LCD_data
A079:
		mov		R16		,	R5
		add		R16		,	R17
		rcall	LCD_data
		ldi		R16		,	'.'
		rcall	LCD_data
		mov		R16		,	R4
		add		R16		,	R17
		rcall	LCD_data
		mov		R16		,	R3
		add		R16		,	R17
		rcall	LCD_data
		mov		R16		,	R2
		add		R16		,	R17
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'V'
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		rjmp	A085
A080:										; MR 255.50  VDC
		rcall	LCD_line1
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'U'
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'='
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		mov		R16		,	R6				; fade out the first leading 0
		cpi		R16		,	0
		brne	A081
		ldi		R16		,	' '
		rcall	LCD_data
		rjmp	A082
A081:
		mov		R16		,	R6
		add		R16		,	R17
		rcall	LCD_data
A082:
		mov		R16		,	R6				; if the first leading number > 0, then
		cpi		R16		,	0				; do not fade out the second leading number
		brne	A083
		mov		R16		,	R5				; fade out the second leading number
		cpi		R16		,	0
		brne	A083
		ldi		R16		,	' '
		rcall	LCD_data
		rjmp	A084
A083:
		mov		R16		,	R5
		add		R16		,	R17
		rcall	LCD_data
A084:
		mov		R16		,	R4
		add		R16		,	R17
		rcall	LCD_data
		ldi		R16		,	'.'
		rcall	LCD_data
		mov		R16		,	R3
		add		R16		,	R17
		rcall	LCD_data
		mov		R16		,	R2
		add		R16		,	R17
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	'V'
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		ldi		R16		,	' '
		rcall	LCD_data
		rjmp	A085
A085:
		pop		R17							; has been pushed in A069

		mov		R16		,	R7
		cpi		R16		,	1
		breq	A086
		cpi		R16		,	2
		breq	A086
		cpi		R16		,	3
		breq	A086
		cpi		R16		,	4
		breq	A086
		cpi		R16		,	5
		breq	A086
		cpi		R16		,	6
		breq	A087
		cpi		R16		,	7
		breq	A087
		cpi		R16		,	8
		breq	A087
		cpi		R16		,	9
		breq	A087
		cpi		R16		,	10
		breq	A087
A086:
		ldi		R16		,	14
		rcall	LCD_goto
		ldi		R16		,	'D'
		rcall	LCD_data
		ldi		R16		,	'C'
		rcall	LCD_data
		mov		R16		,	R10				;
		cpi		R16		,	1				; R10 = 1: LOW BATTERY
		breq	A088					;
		rjmp	A089
A087:
		ldi		R16		,	14
		rcall	LCD_goto
		ldi		R16		,	'A'
		rcall	LCD_data
		ldi		R16		,	'C'
		rcall	LCD_data
		mov		R16		,	R10				;
		cpi		R16		,	1				; R10 = 1: LOW BATTERY
		breq	A088					;
		rjmp	A089
A088:
		ldi		ZL		,	LOW(Text05_LowBat*2)
		ldi		ZH		,	HIGH(Text05_LowBat*2)
		rcall	H201
		ldi		R16		,	10
		rcall	Wait
		rjmp	A089

;-----------------------------------------------------------------------------------------
; Display of the fields, bars and blanks in LCD_line2

A089:
		push	R19							; R19: Anzahl der Felder in LCD_line2
		push	R20							; R20: Anzahl der Balken in LCD_line2
		push	R21							; R21: Anzahl der Blanks in LCD_line2
											; popped after Loop806
;-----------------------------------------------------------------------------------------
A090:										; R24 =  1, 0.0025 bis 0.0300 VDC
		cpi		R24		,	1
		brne	A091
		ldi		R19		,	0				;  0 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	15				; 15 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A091:										; R24 =  2, 0.0325 bis 0.0625 VDC
		cpi		R24		,	2
		brne	A092
		ldi		R19		,	0				;  0 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	15				; 15 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A092:										; R24 =  3, 0.0625 bis 0.0950 VDC
		cpi		R24		,	3
		brne	A093
		ldi		R19		,	0				;  0 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	15				; 15 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A093:										; R24 =  4, 0.0957 bis 0.1275 VDC
		cpi		R24		,	4
		brne	A094
		ldi		R19		,	0				;  0 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	15				; 15 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A094:										; R24 =  5, 0.1300 bis 0.1575 VDC
		cpi		R24		,	5
		brne	A095
		ldi		R19		,	0				;  0 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	15				; 15 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A095:										; R24 =  6, 0.1600 bis 0.1900 VDC
		cpi		R24		,	6
		brne	A096
		ldi		R19		,	1				;  1 Feld
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	14				; 14 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A096:										; R24 =  7, 0.1925 bis 0.2225 VDC
		cpi		R24		,	7
		brne	A097
		ldi		R19		,	1				;  1 Feld
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	14				; 14 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A097:										; R24 =  8, 0.2250 bis 0.2550 VDC
		cpi		R24		,	8
		brne	A098
		ldi		R19		,	1				;  1 Feld
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	14				; 14 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A098:										; R24 =  9, 0.2575 bis 0.2875 VDC
		cpi		R24		,	9
		brne	A099
		ldi		R19		,	1				;  1 Feld
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	14				; 14 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A099:										; R24 = 10, 0.2900 bis 0.3175 VDC
		cpi		R24		,	10
		brne	A100
		ldi		R19		,	1				;  1 Feld
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	14				; 14 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A100:										; R24 = 11, 0.3200 bis 0.3500 VDC
		cpi		R24		,	11
		brne	A101
		ldi		R19		,	2				;  2 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	13				; 13 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A101:										; R24 = 12, 0.3525 bis 0.3825 VDC
		cpi		R24		,	12
		brne	A102
		ldi		R19		,	2				;  2 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	13				; 13 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A102:										; R24 = 13, 0.3850 bis 0.4150 VDC
		cpi		R24		,	13
		brne	A103
		ldi		R19		,	2				;  2 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	13				; 13 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A103:										; R24 = 14, 0.4175 bis 0.4475 VDC
		cpi		R24		,	14
		brne	A104
		ldi		R19		,	2				;  2 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	13				; 13 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A104:										; R24 = 15, 0.4500 bis 0.4775 VDC
		cpi		R24		,	15
		brne	A105
		ldi		R19		,	2				;  2 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	13				; 13 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A105:										; R24 = 16, 0.4800 bis 0.5100 VDC
		cpi		R24		,	16
		brne	A106
		ldi		R19		,	3				;  3 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	12				; 12 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A106:										; R24 = 17, 0.5125 bis 0.5425 VDC
		cpi		R24		,	17
		brne	A107
		ldi		R19		,	3				;  3 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	12				; 12 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A107:										; R24 = 18, 0.5450 bis 0.5750 VDC
		cpi		R24		,	18
		brne	A108
		ldi		R19		,	3				;  3 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	12				; 12 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A108:										; R24 = 19, 0.5775 bis 0.6075 VDC
		cpi		R24		,	19
		brne	A109
		ldi		R19		,	3				;  3 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	12				; 12 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A109:										; R24 = 20, 0.6100 bis 0.6375 VDC
		cpi		R24		,	20
		brne	A110
		ldi		R19		,	3				;  3 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	12				; 12 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A110:										; R24 = 21, 0.6400 bis 0.6700 VDC
		cpi		R24		,	21
		brne	A111
		ldi		R19		,	4				;  4 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	11				; 11 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A111:										; R24 = 22, 0.6725 bis 0.7025 VDC
		cpi		R24		,	22
		brne	A112
		ldi		R19		,	4				;  4 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	11				; 11 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A112:										; R24 = 23, 0.7050 bis 0.7350 VDC
		cpi		R24		,	23
		brne	A113
		ldi		R19		,	4				;  4 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	11				; 11 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A113:										; R24 = 24, 0.7375 bis 0.7675 VDC
		cpi		R24		,	24
		brne	A114
		ldi		R19		,	4				;  4 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	11				; 11 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A114:										; R24 = 25, 0.7700 bis 0.7975 VDC
		cpi		R24		,	25
		brne	A115
		ldi		R19		,	4				;  4 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	11				; 11 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A115:										; R24 = 26, 0.8000 bis 0.8300 VDC
		cpi		R24		,	26
		brne	A116
		ldi		R19		,	5				;  5 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	10				; 10 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A116:										; R24 = 27, 0.8325 bis 0.8625 VDC
		cpi		R24		,	27
		brne	A117
		ldi		R19		,	5				;  5 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	10				; 10 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A117:										; R24 = 28, 0.8650 bis 0.8950 VDC
		cpi		R24		,	28
		brne	A118
		ldi		R19		,	5				;  5 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	10				; 10 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A118:										; R24 = 29, 0.8975 bis 0.9275 VDC
		cpi		R24		,	29
		brne	A120
		ldi		R19		,	5				;  5 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	10				; 10 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A120:										; R24 = 30, 0.9300 bis 0.9575 VDC
		cpi		R24		,	30
		brne	A121
		ldi		R19		,	5				;  5 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	10				; 10 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A121:										; R24 = 31, 0.9600 bis 0.9900 VDC
		cpi		R24		,	31
		brne	A122
		ldi		R19		,	6				;  6 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	9				;  9 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A122:										; R24 = 32, 0.9925 bis 1.0225 VDC
		cpi		R24		,	32
		brne	A123
		ldi		R19		,	6				;  6 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	9				;  9 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A123:										; R24 = 33, 1.0250 bis 1.0550 VDC
		cpi		R24		,	33
		brne	A124
		ldi		R19		,	6				;  6 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	9				;  9 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A124:										; R24 = 34, 1.0575 bis 1.0875 VDC
		cpi		R24		,	34
		brne	A125
		ldi		R19		,	6				;  6 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	9				;  9 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A125:										; R24 = 35, 1.0900 bis 1.1175 VDC
		cpi		R24		,	35
		brne	A126
		ldi		R19		,	6				;  6 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	9				;  9 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A126:										; R24 = 36, 1.1200 bis 1.1500 VDC
		cpi		R24		,	36
		brne	A128
		ldi		R19		,	7				;  7 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	8				;  8 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A128:										; R24 = 37, 1.1525 bis 1.1825 VDC
		cpi		R24		,	37
		brne	A129
		ldi		R19		,	7				;  7 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	8				;  8 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A129:										; R24 = 38, 1.1850 bis 1.2150 VDC
		cpi		R24		,	38
		brne	A130
		ldi		R19		,	7				;  7 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	8				;  8 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A130:										; R24 = 39, 1.2175 bis 1.2475 VDC
		cpi		R24		,	39
		brne	A131
		ldi		R19		,	7				;  7 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	8				;  8 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A131:										; R24 = 40, 1.2500 bis 1.2775 VDC
		cpi		R24		,	40
		brne	A132
		ldi		R19		,	7				;  7 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	8				;  8 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A132:										; R24 = 41, 1.2800 bis 1.3100 VDC
		cpi		R24		,	41
		brne	A133
		ldi		R19		,	8				;  8 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	7				;  7 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A133:										; R24 = 42, 1.3125 bis 1.3425 VDC
		cpi		R24		,	42
		brne	A134
		ldi		R19		,	8				;  8 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	7				;  7 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A134:										; R24 = 43, 1.3450 bis 1.3750 VDC
		cpi		R24		,	43
		brne	A135
		ldi		R19		,	8				;  8 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	7				;  7 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A135:										; R24 = 44, 1.3775 bis 1.4075 VDC
		cpi		R24		,	44
		brne	A136
		ldi		R19		,	8				;  8 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	7				;  7 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A136:										; R24 = 45, 1.4100 bis 1.4375 VDC
		cpi		R24		,	45
		brne	A137
		ldi		R19		,	8				;  8 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	7				;  7 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A137:										; R24 = 46, 1.4400 bis 1.4700 VDC
		cpi		R24		,	46
		brne	A138
		ldi		R19		,	9				;  9 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	6				;  6 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A138:										; R24 = 47, 1.4725 bis 1.5025 VDC
		cpi		R24		,	47
		brne	A139
		ldi		R19		,	9				;  9 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	6				;  6 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A139:										; R24 = 48, 1.5050 bis 1.5350 VDC
		cpi		R24		,	48
		brne	A140
		ldi		R19		,	9				;  9 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	6				;  6 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A140:										; R24 = 49, 1.5375 bis 1.5675 VDC
		cpi		R24		,	49
		brne	A141
		ldi		R19		,	9				;  9 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	6				;  6 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A141:										; R24 = 50, 1.5700 bis 1.5975 VDC
		cpi		R24		,	50
		brne	A142
		ldi		R19		,	9				;  9 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	6				;  6 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A142:										; R24 = 51, 1.6000 bis 1.6300 VDC
		cpi		R24		,	51
		brne	A143
		ldi		R19		,	10				; 10 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	5				;  5 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A143:										; R24 = 52, 1.5325 bis 1.6625 VDC
		cpi		R24		,	52
		brne	A144
		ldi		R19		,	10				; 10 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	5				;  5 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A144:										; R24 = 53, 1.6650 bis 1.6950 VDC
		cpi		R24		,	53
		brne	A145
		ldi		R19		,	10				; 10 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	5				;  5 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A145:										; R24 = 54, 1.6975 bis 1.7275 VDC
		cpi		R24		,	54
		brne	A146
		ldi		R19		,	10				; 10 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	5				;  5 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A146:										; R24 = 55, 1.7300 bis 1.7575 VDC
		cpi		R24		,	55
		brne	A147
		ldi		R19		,	10				; 10 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	5				;  5 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A147:										; R24 = 56, 1.7600 bis 1.7900 VDC
		cpi		R24		,	56
		brne	A148
		ldi		R19		,	11				; 11 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	4				;  4 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A148:										; R24 = 57, 1.7925 bis 1.8225 VDC
		cpi		R24		,	57
		brne	A149
		ldi		R19		,	11				; 11 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	4				;  4 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A149:										; R24 = 58, 1.8250 bis 1.8550 VDC
		cpi		R24		,	58
		brne	A150
		ldi		R19		,	11				; 11 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	4				;  4 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A150:										; R24 = 59, 1.8575 bis 1.8875 VDC
		cpi		R24		,	59
		brne	A151
		ldi		R19		,	11				; 11 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	4				;  4 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A151:										; R24 = 60, 1.8900 bis 1.9175 VDC
		cpi		R24		,	60
		brne	A152
		ldi		R19		,	11				; 11 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	4				;  4 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A152:										; R24 = 61, 1.9200 bis 1.9500 VDC
		cpi		R24		,	61
		brne	A153
		ldi		R19		,	12				; 12 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	3				;  3 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A153:										; R24 = 62, 1.9525 bis 1.9825 VDC
		cpi		R24		,	62
		brne	A154
		ldi		R19		,	12				; 12 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	3				;  3 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A154:										; R24 = 63, 1.9850 bis 2.0150 VDC
		cpi		R24		,	63
		brne	A155
		ldi		R19		,	12				; 12 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	3				;  3 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A155:										; R24 = 64, 2.0175 bis 2.0475 VDC
		cpi		R24		,	64
		brne	A156
		ldi		R19		,	12				; 12 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	3				;  3 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A156:										; R24 = 65, 2.0500 bis 2.0775 VDC
		cpi		R24		,	65
		brne	A157
		ldi		R19		,	12				; 12 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	3				;  3 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A157:										; R24 = 66, 2.0800 bis 2.1100 VDC
		cpi		R24		,	66
		brne	A158
		ldi		R19		,	13				; 13 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	2				;  2 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A158:										; R24 = 67, 2.1125 bis 2.1425 VDC
		cpi		R24		,	67
		brne	A159
		ldi		R19		,	13				; 13 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	2				;  2 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A159:										; R24 = 68, 2.1450 bis 2.1750 VDC
		cpi		R24		,	68
		brne	A160
		ldi		R19		,	13				; 13 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	2				;  2 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A160:										; R24 = 69, 2.1775 bis 2.2075 VDC
		cpi		R24		,	69
		brne	A161
		ldi		R19		,	13				; 13 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	2				;  2 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A161:										; R24 = 70, 2.2100 bis 2.2375 VDC
		cpi		R24		,	70
		brne	A162
		ldi		R19		,	13				; 13 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	2				;  2 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A162:										; R24 = 71, 2.2400 bis 2.2700 VDC
		cpi		R24		,	71
		brne	A163
		ldi		R19		,	14				; 14 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	1				;  1 Blank
		rjmp	A172
;-----------------------------------------------------------------------------------------
A163:										; R24 = 72, 2.2725 bis 2.3025 VDC
		cpi		R24		,	72
		brne	A164
		ldi		R19		,	14				; 14 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	1				;  1 Blank
		rjmp	A172
;-----------------------------------------------------------------------------------------
A164:										; R24 = 73, 2.3050 bis 2.3350 VDC
		cpi		R24		,	73
		brne	A165
		ldi		R19		,	14				; 14 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	1				;  1 Blank
		rjmp	A172
;-----------------------------------------------------------------------------------------
A165:										; R24 = 74, 2.3375 bis 2.3675 VDC
		cpi		R24		,	74
		brne	A166
		ldi		R19		,	14				; 14 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	1				;  1 Blank
		rjmp	A172
;-----------------------------------------------------------------------------------------
A166:										; R24 = 75, 2.3700 bis 2.3975 VDC
		cpi		R24		,	75
		brne	A167
		ldi		R19		,	14				; 14 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	1				;  1 Blank
		rjmp	A172
;-----------------------------------------------------------------------------------------
A167:										; R24 = 76, 2.4000 bis 2.4300 VDC
		cpi		R24		,	76
		brne	A168
		ldi		R19		,	15				; 15 Felder
		ldi		R20		,	1				;  1 Balken
		ldi		R21		,	0				;  0 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A168:										; R24 = 77, 2.4325 bis 2.4625 VDC
		cpi		R24		,	77
		brne	A169
		ldi		R19		,	15				; 15 Felder
		ldi		R20		,	2				;  2 Balken
		ldi		R21		,	0				;  0 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A169:										; R24 = 78, 2.4650 bis 2.4950 VDC
		cpi		R24		,	78
		brne	A170
		ldi		R19		,	15				; 15 Felder
		ldi		R20		,	3				;  3 Balken
		ldi		R21		,	0				;  0 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A170:										; R24 = 79, 2.4975 bis 2.5275 VDC
		cpi		R24		,	79
		brne	A171
		ldi		R19		,	15				; 15 Felder
		ldi		R20		,	4				;  4 Balken
		ldi		R21		,	0				;  0 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
A171:										; R24 = 80, 2.5300 bis 2.5575 VDC
		ldi		R19		,	15				; 15 Felder
		ldi		R20		,	5				;  5 Balken
		ldi		R21		,	0				;  0 Blanks
		rjmp	A172
;-----------------------------------------------------------------------------------------
; Ausgabe LCD Zeile 2: Lschen der LCD Zeile 2 durch Ausgabe von Blanks wenn R22 = 0 und R23 = 0

A172:
		mov		R16		,	R10
		cpi		R16		,	1				; Battery Low ?
		breq	A173
		cpi		R16		,	2				; Continuity Test ?
		breq	A173
		cpi		R16		,	3				; Overflow ?
		breq	A173
		cpi		R16		,	4				; Calibration Mode ?
		breq	A173
		rjmp	A174
A173:										; dann in Line2 nichts anderes ausgeben
		rjmp	A187
A174:
		tst		R23							; 
		brne	A177						; R23 > 0 oder/und R22 > 0 ?
		tst		R22							; dann ist U_Mess > 0
		brne	A177						; und Balken werden in der LCD Zeile 2 ausgegeben
A175:
		ldi		ZL		,	LOW(Text06_LineEmpty*2)
		ldi		ZH		,	HIGH(Text06_LineEmpty*2)
		rcall	H201
A176:
		pop		R21							; pushed after label A089
		pop		R20
		pop		R19
		rjmp	Loop
A177:
		cbr		R19		,	0b11000000
		cbi		PORTD	,	4				; RS
		cbi		PORTD	,	5				; R/W
		ldi		R16		,	0b01000000
		rcall	LCD_cmd
		sbi		PORTD	,	4				; RS
		sbi		PORTD	,	5				; R/W
		rcall	LCD_line2
A178:										; Felder, 0 ... 15
		cpi		R19		,	0
		breq	A179
		ldi		R16		,	5
		rcall	LCD_data
		dec		R19
		tst		R19
		brne	A178
A179:										; Balken, 1 ... 5
		cpi		R20		,	1
		breq	A180
		cpi		R20		,	2
		breq	A181
		cpi		R20		,	3
		breq	A182
		cpi		R20		,	4
		breq	A183
		cpi		R20		,	5
		breq	A184
A180:										; 1 Balken
		ldi		R16		,	1
		ldi		R16		,	0b00000001
		rcall	LCD_data
		rjmp	A185
A181:										; 2 Balken
		ldi		R16		,	2
		ldi		R16		,	0b00000010
		rcall	LCD_data
		rjmp	A185
A182:										; 3 Balken
		ldi		R16		,	3
		ldi		R16		,	0b00000011
		rcall	LCD_data
		rjmp	A185
A183:										; 4 Balken
		ldi		R16		,	4
		ldi		R16		,	0b00000100
		rcall	LCD_data
		rjmp	A185
A184:										; 5 Balken
		ldi		R16		,	5
		ldi		R16		,	0b00000101
		rcall	LCD_data
		rjmp	A185
A185:										; Blanks, 15 ... 0
		cpi		R21		,	0
		breq	A186
		ldi		R16		,	0
		rcall	LCD_data
		dec		R21
		tst		R21
		brne	A185
A186:
		ldi		R16		,	10
		rcall	Wait
A187:
		pop		R21							; pushed after label A089
		pop		R20
		pop		R19

		rjmp	Loop

;-----------------------------------------------------------------------------------------
; Wartezeit 0.1 ms bei 4 MHz
wait01ms:
		push	R16
		push	R17
		ldi		R17		,	1				; fr 0.1 ms = 1
w01ms1:
		ldi		R16		,	123
w01ms2:
		dec		R16
		brne	w01ms2
		dec		R17
		brne	w01ms1
		pop		R17
		pop		R16
		ret

;-----------------------------------------------------------------------------------------
; Wartezeit 0.5 ms bei 4 MHz
wait05ms:
		push	R16
		push	R17
		ldi		R17		,	4				; fr 1 ms = 10
w05ms1:
		ldi		R16		,	123
w05ms2:
		dec		R16
		brne	w05ms2
		dec		R17
		brne	w05ms1
		pop		R17
		pop		R16
		ret

;-----------------------------------------------------------------------------------------
; Wartezeit 5 ms bei 4 MHz
wait5ms:
		push	R16
		push	R17
		ldi		R17		,	50				; fr 1 ms = 10
w5ms1:
		ldi		R16		,	123
w5ms2:
		dec		R16
		brne	w5ms2
		dec		R17
		brne	w5ms1
		pop		R17
		pop		R16
		ret

;-----------------------------------------------------------------------------------------
; Wartezeit: R16 = 1 ==> 0.01 s, R16 = 255 ==> 2.55 s bei = 3.686 MHz

Wait:
		push	R17
		push	R18

		cpi		R16	,	0
		breq	WLoop0
WLoop1:
		ldi		R17		,	0b01101110
WLoop2:
		ldi		R18		,	0b01101110
WLoop3:
		dec		R18
		brne	WLoop3
		nop
		nop
		dec		R17
		brne	WLoop2
		dec		R16
		brne	WLoop1
WLoop0:
		pop		R18
		pop		R17
		ret

;-----------------------------------------------------------------------------------------
;****************************************************************** Anfang Initialisierung
; Initialisierung des LCD DOGM162W-A mit ST7036 Dot Matrix LCD Controller/Driver

LCD_init:
		push	R18
		ldi		R18		,   50
PowerUpWait:
		rcall	wait5ms						; Power Up Wait Time
		dec		R18							; 250 ms
		brne	PowerUpWait					;
		pop		R18

;-----------------------------------------------------------------------------------------
; Function Set 01: 3 Mal den 8-Bit Mode ausgeben

		ldi		R16		,	0b00110000		; 8 Bit Mode
		swap	R16                         ; 0b00000011 wegen 4 Bit Verdrahtung
											; RS  = Low = Command
											; R/W = Low = Read
		out		PORTD	,	R16				; must be set 3 times, why?
;-----------------------------------------------------------------------------------------
; Function Set 01: Das 1. Mal

		rcall	LCD_enable					; Impuls von E = MPU D.6 Pin 12
		rcall	wait5ms						; Wait 5ms
;-----------------------------------------------------------------------------------------
; Function Set 02: Das 2. Mal

		rcall	LCD_enable					; Impuls von E = MPU D.6 Pin 12
		rcall	wait5ms						;
;-----------------------------------------------------------------------------------------
; Function Set 03: Das 3. Mal

		rcall	LCD_enable					; Impuls von E = MPU D.6 Pin 12
		rcall	wait5ms						;

;-----------------------------------------------------------------------------------------
; Function Set 04: Den 4-Bit Mode einschalten

		ldi		R16		,	0b00100000		; 4 Bit Mode, D.4 = Low = Command
		swap	R16                         ; 0b00000010 wegen 4 Bit Verdrahtung
		out		PORTD	,	R16				;
		rcall	LCD_enable					; Impuls von E = MPU D.6 Pin 12
		rcall	wait5ms						;

;-----------------------------------------------------------------------------------------
; Function Set 05: 4-Bit Mode, 2 Lines, 5x8 Dots, Extension Instructions, CGRam not available

		ldi		R16		,	0b00101001
		rcall	LCD_cmd

;-----------------------------------------------------------------------------------------
; Function Set 06: Bias, 2 Lines

		ldi		R16		,	0b00010100		; BIAS
		rcall	LCD_cmd						;

;-----------------------------------------------------------------------------------------
; Function Set 07: Contrast Set (Low Byte)

		ldi		R16		,	0b01111000		; Contrast Set
		rcall	LCD_cmd	

;-----------------------------------------------------------------------------------------
; Function Set 08: ICON/Power/Contrast (High Byte)

		ldi		R16		,	0b01011110		; ICON/Power/Contrast
		rcall	LCD_cmd						;

;-----------------------------------------------------------------------------------------
; Function Set 09: Follower Control

		ldi		R16		,	0b01101010		; Follower Control
		rcall	LCD_cmd						;

;-----------------------------------------------------------------------------------------
; Function Set 10: 4-Bit Mode, 2 Lines, 5x8 Dots, Normal Instructions, CGRam available

		ldi		R16		,	0b00101000
		rcall	LCD_cmd

;-----------------------------------------------------------------------------------------
; Function Set 11: Display On

		ldi		R16		,	0b00001100		; Display On
		rcall	LCD_cmd						;

;-----------------------------------------------------------------------------------------
; Function Set 12: Clear Display

		ldi		R16		,	0b00001101		; Clear Display
		rcall	LCD_cmd						;

;-----------------------------------------------------------------------------------------
; Functin Set 13: Entry Mode Set

		ldi		R16		,	0b00001100		; Entry Mode Set
		rcall	LCD_cmd

;-----------------------------------------------------------------------------------------
; The End

		ret

;******************************************************************** Ende Initialisierung
;-----------------------------------------------------------------------------------------
; Ausgabe von Daten zur Anzeige im LCD, sendet ein Datenbyte

LCD_data:
		push	R17
		mov		R17		,	R16				; "Sicherungskopie" fr das 2. Nibble
		swap	R16							; Nibbles vertauschen
		andi	R16		,	0b00001111		; obereses Nibble auf Null setzen
		sbr		R16		,	0b00010000		; RS auf 1
		out		PORTD	,	R16				; 1. Nibble plus RS ausgeben
		rcall	LCD_enable					; 1. Nibble plus RS bernehmen
											; 2. Nibble kein swap, da es in R17 schon an
											; der richtigen Stelle steht
		andi	R17		,	0b00001111		; oberes Nibble auf Null setzen
		sbr		R17		,	0b00010000		; RS auf 1
		out		PORTD	,	R17				; 2. Nibble plus RS ausgeben
		rcall	LCD_enable					; 2. Nibble plus RS bernehmen
		rcall	LCD_busy					; Busy Flag prfen
		pop		R17
		ret

;-----------------------------------------------------------------------------------------
; Ausgabe von Kommandos ans LCD, wie LCD_data aber RS = 0

LCD_cmd:
		push	R17
		mov		R17		,	R16				; "Sicherungskopie" fr das 2. Nibble
		swap	R16							; Nibbles vertauschen
		andi	R16		,	0b00001111		; obereses Nibble auf Null setzen
		sbr		R16		,	0b00000000		; RS auf 0 (bzw. nicht auf 1)
		out		PORTD	,	R16				; 1. Nibble ausgeben
		rcall	LCD_enable					; 1. Nibble bernehmen
											; 2. Nibble kein swap, da es in R17 schon an
											; der richtigen Stelle steht
		andi	R17		,	0b00001111		; oberes Nibble auf Null setzen
		sbr		R17		,	0b00000000		; RS auf 0 (bzw. nicht auf 1)
		out		PORTD	,	R17				; 2. Nibble ausgeben
		rcall	LCD_enable					; 2. Nibble bernehmen
		rcall	LCD_busy					; Busy Flag prfen
		pop		R17
		ret

;-----------------------------------------------------------------------------------------
LCD_enable:
		sbi		PORTD	,	6				; Enable High
		nop									; kurz warten
		nop
		nop
		cbi		PORTD	,	6				; Enable Low
		ret

;-----------------------------------------------------------------------------------------
; Busy Flag prfen

LCD_busy:
		push	R16
		ldi		R16		,	0b11110000		; Disable Data Bit Outputs
		out		DDRD	,	R16
		ldi		R16		,	0b00000000		; Clear all outputs
		out		PORTD	,	R16
LCD_busy1:
		ldi		R16		,	0b00100000		; Enable only read bit
		out		PORTD	,	R16
		sbi		PORTD	,	6				; Raise the Enable signal
		nop									; kurz warten
		nop
		in		R16		,	PIND			; Read the current values
		cbi		PORTD	,	6				; Disable the Enable signal
		rcall	LCD_enable					; Puls the Enable (the second nibble is discarded)
		swap	R16							; Busy flag von R16.3 nach R16.7
		sbrc	R16		,	7				; Check busy flag
		rjmp	LCD_busy1					; 
		ldi		R16		,	0b11111111		; Enable all outputs
		out		DDRD	,	R16
		pop		R16
		ret

;-----------------------------------------------------------------------------------------
LCD_clear:
		ldi		R16		,	0b00000001		; Display lschen
		rcall	LCD_cmd
		ret

;-----------------------------------------------------------------------------------------
LCD_home:
		ldi		R16		,	0b00000010		; Display Cursor HOME
		rcall	LCD_cmd
		ret

;-----------------------------------------------------------------------------------------
LCD_off:
		ldi		R16		,	0b00001000
		rcall	LCD_cmd
		ret

;-----------------------------------------------------------------------------------------
LCD_on:
		ldi		R16		,	0b00001110
		rcall	LCD_cmd
		ret

;-----------------------------------------------------------------------------------------
LCD_line1:
		ldi		R16		,	0b10000000		; DRAM auf Adresse 0x00
		rcall	LCD_cmd
		ret

;-----------------------------------------------------------------------------------------
LCD_line2:
		ldi		R16		,	0b11000000		; DRAM auf Adresse 0x40
		rcall	LCD_cmd
		ret

;-----------------------------------------------------------------------------------------
; Goto R16 = Adresse (Zeile 1 = 0...15, Zeile 2 = 64...79
LCD_goto:
		ori		R16		,	0b10000000		; Goto DRAM auf Adresse R16
											; ORI - Logical OR with Immediate
		rcall	LCD_cmd
		ret

;-----------------------------------------------------------------------------------------
LCD_CUL:
		ldi		R16		,	0b00010000		; Cursor um eine Poation nach links
		rcall	LCD_cmd
		ret

;-----------------------------------------------------------------------------------------
LCD_CUR:
		ldi		R16		,	0b00010100		; Cursor um eine Postion nach rechts
		rcall	LCD_cmd
		ret

;-----------------------------------------------------------------------------------------
; R16 numerisch augeben, d.h. als Ziffern 0 bis 255, dezimal

LCD_number:
		push	R16
		push	R17
		mov		R17		,	R16
		ldi		R16		,	'0'-1
LCD_number1:
		inc		R16
		subi	R17		,	100
		brcc	LCD_number1
		subi	R17		,	-100
		rcall	LCD_data
		ldi		R16		,	'0'-1
LCD_number2:
		inc		R16
		subi	R17		,	10
		brcc	LCD_number2
		subi	R17		,	-10
		rcall	LCD_data
LCD_number3:
		ldi		R16		,	'0'
		add		R16		,	R17
		rcall	LCD_data
		pop		R17
		pop		R16
		ret

;-----------------------------------------------------------------------------------------
onADC:
		push	R0
		push	R1
		push	R16
		push	R17
		push	R18
		push	R20
		push	R21
		push	R25
		push	R26
		in		R16		,	ADCL			; 10 Bit ADC-Daten holen
		in		R17		,	ADCH
onADC1:
		ldi		R20		,	25				; Multiplikator Low  = 25
		ldi		R21		,	0				; Multiplikator High =  0
		rcall	B001
		rcall	C001						; Multiplikation durchfhren
		cpi		R27		,	0				; Bar Graph None,   Kalibriermodus Aus
		breq	onADC2
		cpi		R27		,	1				; Bar Graph Type 1, Kalibriermodus Aus
		breq	onADC2
		cpi		R27		,	2				; Bar Graph Type 2, Kalibriermodus Aus
		breq	onADC2
		cpi		R27		,	3				; Calibration Mode, Kalibriermodus Ein
		breq	onADC3
onADC2:
		rcall	E001						; Korekturberechnungen Ein
		rjmp	onADC4						; keine Bereichsanzeige
onADC3:
		rcall	F001						; Korekturberechnungen Aus
		rjmp	onADC4						; mit Bereichsanzeige
onADC4:
		rcall	D001						; Dezimalzahl ermitteln
		pop		R26
		pop		R25
		pop		R21
		pop		R20
		pop		R18
		pop		R17
		pop		R16
		pop		R1
		pop		R0
		reti

;-----------------------------------------------------------------------------------------
; Maximum, Minimum

B001:
;		cp		R22		,	R16
;		cpc		R23		,	R17
;		brlo	B002
;		rjmp	B003
;B002:
		mov		R22		,	R16
		mov		R23		,	R17
;B003:
		ret

;-----------------------------------------------------------------------------------------
; Multiplikation von 2 Binrzahlen zu je 16 Bit
;
C001:
		mul		R16		,	R20
		movw	R3:R2	,	R1:R0
		mul		R17		,	R21
		movw	R5:R4	,	R1:R0
		mul		R16		,	R21
		add		R3		,	R0
		adc		R4		,	R1
		clr		R0
		adc		R5		,	R0
		mul		R17		,	R20
		add		R3		,	R0
		adc		R4		,	R1
		clr		R0
		adc		R5		,	R0
		mov		R18		,	R3
		mov		R17		,	R2
		ret

;-----------------------------------------------------------------------------------------
; Dezimalzahl aus einer Binrzahl mit 16 Bit ermitteln

D001:
		mov		R0		,	R17
		mov		R1		,	R18

		clr		R2
		clr		R3
		clr		R4
		clr		R5
		sub		R6		,	R6
		tst		R0
		cpc		R1		,	R2
		breq	D008
		ldi		R16		,	9
		ldi		R17		,	17
D002:
		dec		R17
		lsl		R0
		rol		R1
		brcc	D002
D003:
		adc		R2		,	R2
		cp		R16		,	R2
		brcc	D004
		sbc		R2		,	R16
		sec
D004:
		adc		R3		,	R3
		cp		R16		,	R3
		brcc	D005
		sbc		R3		,	R16
		sec
D005:
		adc		R4		,	R4
		cp		R16		,	R4
		brcc	D006
		sbc		R4		,	R16
		sec
D006:
		adc		R5		,	R5
		cp		R16		,	R5
		brcc	D007
		sbc		R5		,	R16
		sec
D007:
		adc		R6		,	R6
		lsl		R0
		rol		R1
		dec		R17
		brne	D003

; Prfung, ob die Dezimalzahl gleich oder grer   2   5   5   7   5  ist
;												  R6  R5  R4  R3  R2

D008:
		mov		R16		,	R6
		cpi		R16		,	2				; R6 = 2 ?
		breq	D009						; dann nach D009
		cpi		R16		,	3				; R6 >=3 ?			| wegen der Korrekturberechnungen
		brsh	D014						; dann nach D014	| im VAC - Bereich knnen
		rjmp	D015						; wenn R6 < 2		  auch Ziffernfolgen grer als
D009:										;					  25575 erreicht werden			
		mov		R16		,	R5				;					  s. Labels E001 bis E428.
		cpi		R16		,	5				; R5 = 5 ?			  Auch dann mu OVERFLOW
		breq	D010						; dann nach D010	  angezeigt werden.
		cpi		R16		,	6				; R5 >=6 ?			|
		brsh	D014						; dann nach D014	|
		rjmp	D015
D010:
		mov		R16		,	R4
		cpi		R16		,	5				; R4 = 5 ?
		breq	D011						; dann nach D011
		cpi		R16		,	6				; R4 >=6 ?			|
		brsh	D014						; dann nach D014	|
		rjmp	D015
D011:
		mov		R16		,	R3
		cpi		R16		,	7				; R3 = 7 ?
		breq	D013						; dann nach D013
		cpi		R16		,	8				; R3 >=8 ?			|
		brsh	D014						; dann nach D014	|
		rjmp	D015
D013:
		mov		R16		,	R2
		cpi		R16		,	5				; R2 = 5 ?
		breq	D014						; dann nach D014
		cpi		R16		,	6				; R6 >=6 ?			|
		brsh	D014						; dann nach D014	|
		rjmp	D015						; wenn R5 < 5
D014:
		rcall	D016						; Overflow
D015:
		ret

;-----------------------------------------------------------------------------------------
; "OVERFLOW" in LCD_line2, falls zutreffend

D016:
		ldi		ZL		,	LOW(Text07_Overflow*2)
		ldi		ZH		,	HIGH(Text07_Overflow*2)
		rcall	H201
		ldi		R16		,	3
		mov		R10		,	R16

		ret

;-----------------------------------------------------------------------------------------
; Korrektur-Berechnungen fr die 4 AC-Mebereiche
;
E001:
		mov		R16		,	R7
		cpi		R16		,	7				; AC 0,25 V
		breq	E002
		cpi		R16		,	8				; AC 2,5  V
		breq	E003
		cpi		R16		,	9				; AC 25   V
		breq	E004
		cpi		R16		,	10				; AC 250  V
		breq	E005
		ret
E002:
		rcall	E101
		ret
E003:
		rcall	E201
		ret
E004:
		rcall	E301
		ret
E005:
		rcall	E401
		ret

;-----------------------------------------------------------------------------------------
; Korrektur-Berechnung des Mebereichs 0,25 VAC
;
E101:
		cpi		R24		,	1
		brsh	E102
		ldi		R25		,	0b00000000		;   0 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E102:
		cpi		R24		,	2
		brsh	E103
		ldi		R25		,	0b01001011		; - 3 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E103:
		cpi		R24		,	4
		brsh	E104
		ldi		R25		,	0b11111010		; -10 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E104:
		cpi		R24		,	7
		brsh	E105
		ldi		R25		,	0b00101100		; -12 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E105:
		cpi		R24		,	10
		brsh	E106
		ldi		R25		,	0b00101100		; -12 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E106:
		cpi		R24		,	13
		brsh	E107
		ldi		R25		,	0b01000101		; -13 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E107:
		cpi		R24		,	16
		brsh	E108
		ldi		R25		,	0b01000101		; -13 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E108:
		cpi		R24		,	19
		brsh	E109
		ldi		R25		,	0b01000101		; -13 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E109:
		cpi		R24		,	23
		brsh	E110
		ldi		R25		,	0b00101100		; -12 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E110:
		cpi		R24		,	26
		brsh	E111
		ldi		R25		,	0b00101100		; -12 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E111:
		cpi		R24		,	29
		brsh	E112
		ldi		R25		,	0b00101100		; -12 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E112:
		cpi		R24		,	32
		brsh	E113
		ldi		R25		,	0b00010011		; -11 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E113:
		cpi		R24		,	35
		brsh	E114
		ldi		R25		,	0b00010011		; -11 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E114:
		cpi		R24		,	38
		brsh	E115
		ldi		R25		,	0b11111010		; -10 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E115:
		cpi		R24		,	42
		brsh	E116
		ldi		R25		,	0b11111010		; -10 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E116:
		cpi		R24		,	45
		brsh	E117
		ldi		R25		,	0b11100001		; - 9 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E117:
		cpi		R24		,	48
		brsh	E118
		ldi		R25		,	0b11001000		; - 8 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E118:
		cpi		R24		,	51
		brsh	E119
		ldi		R25		,	0b10101111		; - 7 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E119:
		cpi		R24		,	54
		brsh	E120
		ldi		R25		,	0b10101111		; - 7 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E120:
		cpi		R24		,	57
		brsh	E122
		ldi		R25		,	0b10010110		; - 6 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E122:
		cpi		R24		,	60
		brsh	E123
		ldi		R25		,	0b10010110		; - 6 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E123:
		cpi		R24		,	64
		brsh	E124
		ldi		R25		,	0b01111101		; - 5 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E124:
		cpi		R24		,	68
		brsh	E125
		ldi		R25		,	0b01100100		; - 4 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E125:
		cpi		R24		,	70
		brsh	E126
		ldi		R25		,	0b01001011		; - 3 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E126:
		cpi		R24		,	74
		brsh	E127
		ldi		R25		,	0b01001011		; - 3 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E127:
		cpi		R24		,	77
		brsh	E128
		ldi		R25		,	0b00110010		; - 2 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E128:
		ldi		R25		,	0b00110010		; - 2 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E129
E129:
		ret

;-----------------------------------------------------------------------------------------
; Korrektur-Berechnung des Mebereichs 2,5 VAC
;
E201:
		cpi		R24		,	1
		brsh	E202
		ldi		R25		,	0b00000000		;   0 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E202:
		cpi		R24		,	2
		brsh	E203
		ldi		R25		,	0b00110010		; - 9 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E203:
		cpi		R24		,	6
		brsh	E204
		ldi		R25		,	0b01011110		; -14 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E204:
		cpi		R24		,	9
		brsh	E205
		ldi		R25		,	0b10101001		; -17 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E205:
		cpi		R24		,	12
		brsh	E206
		ldi		R25		,	0b11000010		; -18 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E206:
		cpi		R24		,	25
		brsh	E207
		ldi		R25		,	0b11000010		; -18 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E207:
		cpi		R24		,	18
		brsh	E208
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E208:
		cpi		R24		,	21
		brsh	E209
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E209:
		cpi		R24		,	24
		brsh	E210
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E210:
		cpi		R24		,	27
		brsh	E211
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E211:
		cpi		R24		,	31
		brsh	E212
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E212:
		cpi		R24		,	34
		brsh	E213
		ldi		R25		,	0b11000010		; -18 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E213:
		cpi		R24		,	37
		brsh	E214
		ldi		R25		,	0b10101001		; -17 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E214:
		cpi		R24		,	40
		brsh	E215
		ldi		R25		,	0b10010000		; -16 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E215:
		cpi		R24		,	43
		brsh	E216
		ldi		R25		,	0b10010000		; -16 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E216:
		cpi		R24		,	46
		brsh	E217
		ldi		R25		,	0b01110111		; -15 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E217:
		cpi		R24		,	50
		brsh	E218
		ldi		R25		,	0b01011110		; -14 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E218:
		cpi		R24		,	53
		brsh	E219
		ldi		R25		,	0b01011110		; -14 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E219:
		cpi		R24		,	56
		brsh	E220
		ldi		R25		,	0b01000101		; -13 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E220:
		cpi		R24		,	59
		brsh	E221
		ldi		R25		,	0b01000101		; -13 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E221:
		cpi		R24		,	62
		brsh	E222
		ldi		R25		,	0b00101100		; -12 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E222:
		cpi		R24		,	65
		brsh	E223
		ldi		R25		,	0b00010011		; -11 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E223:
		cpi		R24		,	69
		brsh	E224
		ldi		R25		,	0b11111010		; -10 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E224:
		cpi		R24		,	72
		brsh	E225
		ldi		R25		,	0b11111010		; -10 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E225:
		cpi		R24		,	75
		brsh	E226
		ldi		R25		,	0b11100001		; - 9 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E226:
		cpi		R24		,	78
		brsh	E227
		ldi		R25		,	0b11001000		; - 8 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E227:
		ldi		R25		,	0b10101111		; - 7 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E228
E228:
		ret

;-----------------------------------------------------------------------------------------
; Korrektur-Berechnung des Mebereichs 25 VAC
;
E301:
		cpi		R24		,	1
		brsh	E302
		ldi		R25		,	0b00000000		;   0 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E302:
		cpi		R24		,	2
		brsh	E303
		ldi		R25		,	0b00000000		;   0 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E303:
		cpi		R24		,	6
		brsh	E304
		ldi		R25		,	0b01110111		; -15 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E304:
		cpi		R24		,	9
		brsh	E305
		ldi		R25		,	0b10101001		; -17 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E305:
		cpi		R24		,	12
		brsh	E306
		ldi		R25		,	0b11000010		; -18 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E306:
		cpi		R24		,	15
		brsh	E307
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E307:
		cpi		R24		,	18
		brsh	E308
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E308:
		cpi		R24		,	21
		brsh	E309
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E309:
		cpi		R24		,	24
		brsh	E310
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E310:
		cpi		R24		,	27
		brsh	E311
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E311:
		cpi		R24		,	31
		brsh	E312
		ldi		R25		,	0b11000010		; -18 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E312:
		cpi		R24		,	34
		brsh	E313
		ldi		R25		,	0b10101001		; -17 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E313:
		cpi		R24		,	37
		brsh	E314
		ldi		R25		,	0b10101001		; -17 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E314:
		cpi		R24		,	40
		brsh	E315
		ldi		R25		,	0b10101001		; -17 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E315:
		cpi		R24		,	43
		brsh	E316
		ldi		R25		,	0b10010000		; -16 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E316:
		cpi		R24		,	46
		brsh	E317
		ldi		R25		,	0b01110111		; -15 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E317:
		cpi		R24		,	50
		brsh	E318
		ldi		R25		,	0b01110111		; -15 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E318:
		cpi		R24		,	53
		brsh	E319
		ldi		R25		,	0b01011110		; -14 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E319:
		cpi		R24		,	56
		brsh	E320
		ldi		R25		,	0b01000101		; -13 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E320:
		cpi		R24		,	59
		brsh	E321
		ldi		R25		,	0b00101100		; -12 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E321:
		cpi		R24		,	62
		brsh	E322
		ldi		R25		,	0b00101100		; -12 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E322:
		cpi		R24		,	65
		brsh	E323
		ldi		R25		,	0b00010011		; -11 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E323:
		cpi		R24		,	69
		brsh	E324
		ldi		R25		,	0b00010011		; -11 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E324:
		cpi		R24		,	72
		brsh	E325
		ldi		R25		,	0b00010011		; -11 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E325:
		cpi		R24		,	75
		brsh	E326
		ldi		R25		,	0b11111010		; -10 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E326:
		cpi		R24		,	78
		brsh	E327
		ldi		R25		,	0b11001000		; - 8 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E327:
		ldi		R25		,	0b10101111		; - 7 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E328
E328:
		ret

;-----------------------------------------------------------------------------------------
; Korrektur-Berechnung des Mebereichs 250 VAC
;
E401:
		cpi		R24		,	1
		brsh	E402
		ldi		R25		,	0b00000000		;   0 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E402:
		cpi		R24		,	2
		brsh	E403
		ldi		R25		,	0b00000000		;   0 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E403:
		cpi		R24		,	5
		brsh	E404
		ldi		R25		,	0b00001101		; -21 Digits
		ldi		R26		,	0b00000010
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E404:
		cpi		R24		,	8
		brsh	E405
		ldi		R25		,	0b01011000		; -24 Digits
		ldi		R26		,	0b00000010
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E405:
		cpi		R24		,	11
		brsh	E406
		ldi		R25		,	0b01011000		; -24 Digits
		ldi		R26		,	0b00000010
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E406:
		cpi		R24		,	14
		brsh	E407
		ldi		R25		,	0b01011000		; -24 Digits
		ldi		R26		,	0b00000010
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E407:
		cpi		R24		,	18
		brsh	E408
		ldi		R25		,	0b01011000		; -24 Digits
		ldi		R26		,	0b00000010
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E408:
		cpi		R24		,	21
		brsh	E409
		ldi		R25		,	0b00111111		; -23 Digits
		ldi		R26		,	0b00000010
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E409:
		cpi		R24		,	24
		brsh	E410
		ldi		R25		,	0b00100110		; -22 Digits
		ldi		R26		,	0b00000010
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E410:
		cpi		R24		,	27
		brsh	E411
		ldi		R25		,	0b00100110		; -22 Digits
		ldi		R26		,	0b00000010
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E411:
		cpi		R24		,	30
		brsh	E412
		ldi		R25		,	0b00001101		; -21 Digits
		ldi		R26		,	0b00000010
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E412:
		cpi		R24		,	33
		brsh	E413
		ldi		R25		,	0b11110100		; -20 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E413:
		cpi		R24		,	37
		brsh	E414
		ldi		R25		,	0b11110100		; -20 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E414:
		cpi		R24		,	40
		brsh	E415
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E415:
		cpi		R24		,	43
		brsh	E416
		ldi		R25		,	0b11011011		; -19 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E416:
		cpi		R24		,	46
		brsh	E417
		ldi		R25		,	0b11000010		; -18 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E417:
		cpi		R24		,	49
		brsh	E418
		ldi		R25		,	0b10101001		; -17 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E418:
		cpi		R24		,	53
		brsh	E419
		ldi		R25		,	0b10010000		; -16 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E419:
		cpi		R24		,	56
		brsh	E420
		ldi		R25		,	0b01110111		; -15 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E420:
		cpi		R24		,	59
		brsh	E421
		ldi		R25		,	0b01011110		; -14 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E421:
		cpi		R24		,	62
		brsh	E422
		ldi		R25		,	0b01011110		; -14 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E422:
		cpi		R24		,	65
		brsh	E423
		ldi		R25		,	0b01000101		; -13 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E423:
		cpi		R24		,	69
		brsh	E424
		ldi		R25		,	0b00101100		; -12 Digits
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E424:
		cpi		R24		,	72
		brsh	E425
		ldi		R25		,	0b00010011		; -11 Digit
		ldi		R26		,	0b00000001
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E425:
		cpi		R24		,	75
		brsh	E426
		ldi		R25		,	0b11111010		; -10 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E426:
		cpi		R24		,	78
		brsh	E427
		ldi		R25		,	0b11100001		; - 9 Digits
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E427:
		ldi		R25		,	0b11001000		; - 8 Digit
		ldi		R26		,	0b00000000
		add		R17		,	R25
		adc		R18		,	R26
		rjmp	E428
E428:
		ret

;-----------------------------------------------------------------------------------------
; Calibration Mode: ASCII-Wert von R24 in LCD_line2 anzeigen

F001:
		mov		R16		,	R10
		cpi		R16		,	1				; Battery Low ?
		breq	F004
		mov		R16		,	R10
		cpi		R16		,	2				; Continuity Test ?
		breq	F004
		mov		R16		,	R10
		cpi		R16		,	3				; Overflow ?
		breq	F004

		ldi		ZL		,	LOW(Text21_Calib*2)
		ldi		ZH		,	HIGH(Text21_Calib*2)
		rcall	LCD_line2
F002:
		lpm		R16		,	Z+
		cpi		R16		,	255
		breq	F003
		rcall	LCD_data
		rjmp	F002
F003:
		ldi		R16		,	77
		rcall	LCD_goto
		mov		R16		,	R24
		rcall	LCD_number
		ldi		R16		,	4				; Calibration Mode
		mov		R10		,	R16
		ldi		R16		,	10
		rcall	Wait
F004:
		ret

;-----------------------------------------------------------------------------------------
; Zeichen ins CGRam des LCD schreiben
;
CG00:
		cpi		R27		,	0				; Bar Graph None
		breq	CG01
		cpi		R27		,	1				; Bar Graph Type 1 "||||| ||||| ||||| |||"
		breq	CG02
		cpi		R27		,	2				; Bar Graph Type 2 "| | | | | | | | | | |"
		breq	CG03
CG01:
		ldi		ZL		,	LOW(Text00_Symbole0*2)
		ldi		ZH		,	HIGH(Text00_Symbole0*2)
		rjmp	CG04
CG02:
		ldi		ZL		,	LOW(Text01_Symbole1*2)
		ldi		ZH		,	HIGH(Text01_Symbole1*2)
		rjmp	CG04
CG03:
		ldi		ZL		,	LOW(Text02_Symbole2*2)
		ldi		ZH		,	HIGH(Text02_Symbole2*2)
		rjmp	CG04
CG04:
		ldi		R16		,	64
		rcall	LCD_cmd
CG05:
		lpm		R16		,	Z+
		cpi		R16		,	255
		breq	CG06
		rcall	LCD_data
		rjmp	CG05
CG06:
		ret

;-----------------------------------------------------------------------------------------
; Timer/Counter0 Kontrolle

onTMR0:
		sbis	PINC	,	0
		rjmp	G001
		sbis	PINC	,	2
		rcall	G002
		reti

;-----------------------------------------------------------------------------------------
; Funktionen mittels der grnen Taste aufrufen: Messbereiche

G001:
		cli
		push	R16
		push	R18
		push	R19
		ldi		R16		,	0
		ldi		R18		,	0
		ldi		R19		,	0
		ldi		ZL		,	LOW(Text09_SelRange*2)
		ldi		ZH		,	HIGH(Text09_SelRange*2)
		rcall	H101							; LCD_line1
		ldi		ZL		,	LOW(Text06_LineEmpty*2)
		ldi		ZH		,	HIGH(Text06_LineEmpty*2)
		rcall	H201							; LCD_line2
		ldi		R16		,	50				; 500 ms warten damit die Taste
		rcall	Wait						; wieder losgelassen wurde
G011:
		inc		R18							; wenn innerhalb von 2 s kein
		ldi		R16		,	1				; Tastendruck erfolgt, dann wird
		rcall	Wait						; die zuletzt gewhlte Funktion
		cpi		R18		,	200				; bernommen
		breq	G012
		sbic	PINC	,	0				; warten auf Tastendruck
		rjmp	G011
		rjmp	G013
G012:
		pop		R19
		pop		R18
		pop		R16
		sei
		rjmp	Loop
G013:
		inc		R19
		cpi		R19		,	1
		breq	G101
		cpi		R19		,	2
		breq	G102
		cpi		R19		,	3
		breq	G103
		cpi		R19		,	4
		breq	G104
		cpi		R19		,	5
		breq	G105
		cpi		R19		,	6
		breq	G106
		cpi		R19		,	7
		breq	G107
		cpi		R19		,	8
		breq	G108
		cpi		R19		,	9
		breq	G109
		cpi		R19		,	10
		breq	G110
		cpi		R19		,	11
		breq	G111
		ldi		R19		,	0
		rjmp	G011
G101:
		rjmp	G121
G102:
		rjmp	G122
G103:
		rjmp	G123
G104:
		rjmp	G124
G105:
		rjmp	G125
G106:
		rjmp	G126
G107:
		rjmp	G127
G108:
		rjmp	G128
G109:
		rjmp	G129
G110:
		rjmp	G130
G111:
		rjmp	G131
G121:
		ldi		ZL		,	LOW(Text10_DCAuto*2)
		ldi		ZH		,	HIGH(Text10_DCAuto*2)
		rcall	H201
		ldi		R16		,	1
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G122:
		ldi		ZL		,	LOW(Text11_DC0_25*2)
		ldi		ZH		,	HIGH(Text11_DC0_25*2)
		rcall	H201
		ldi		R16		,	2
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G123:
		ldi		ZL		,	LOW(Text12_DC2_5*2)
		ldi		ZH		,	HIGH(Text12_DC2_5*2)
		rcall	H201
		ldi		R16		,	3
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G124:
		ldi		ZL		,	LOW(Text13_DC25*2)
		ldi		ZH		,	HIGH(Text13_DC25*2)
		rcall	H201
		ldi		R16		,	4
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G125:
		ldi		ZL		,	LOW(Text14_DC250*2)
		ldi		ZH		,	HIGH(Text14_DC250*2)
		rcall	H201
		ldi		R16		,	5
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G126:
		ldi		ZL		,	LOW(Text15_ACAuto*2)
		ldi		ZH		,	HIGH(Text15_ACAuto*2)
		rcall	H201
		ldi		R16		,	6
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G127:
		ldi		ZL		,	LOW(Text16_AC0_25*2)
		ldi		ZH		,	HIGH(Text16_AC0_25*2)
		rcall	H201
		ldi		R16		,	7
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G128:
		ldi		ZL		,	LOW(Text17_AC2_5*2)
		ldi		ZH		,	HIGH(Text17_AC2_5*2)
		rcall	H201
		ldi		R16		,	8
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G129:
		ldi		ZL		,	LOW(Text18_AC25*2)
		ldi		ZH		,	HIGH(Text18_AC25*2)
		rcall	H201
		ldi		R16		,	9
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G130:
		ldi		ZL		,	LOW(Text19_AC250*2)
		ldi		ZH		,	HIGH(Text19_AC250*2)
		rcall	H201
		ldi		R16		,	10
		mov		R7		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G011
G131:
		ldi		ZL		,	LOW(Text20_MaxMin*2)
		ldi		ZH		,	HIGH(Text20_MaxMin*2)
		rcall	H201
		ldi		R16		,	11
		mov		R7		,	R16
		ldi		R16		,	1
		mov		R9		,	R16				; R9 = 1: MAX MIN On
		ldi		R16		,	50
		rcall	Wait
		rjmp	G121						; DC Auto einschalten

;-----------------------------------------------------------------------------------------
; Funktionen mittels der weien G001 aufrufen: Mode Of The Display Line 2

G002:
		cli
		push	R16
		push	R18
		push	R19
		ldi		R16		,	0
		ldi		R18		,	0
		ldi		R19		,	0
		mov		R9		,	R16				; R9 = 0: MAX MIN Off
		rcall	LCD_line1					; "Select the Mode "
		ldi		ZL		,	LOW(Text22_SelMode*2)
		ldi		ZH		,	HIGH(Text22_SelMode*2)
		rcall	H101
		ldi		R16		,	50				; 500 ms warten damit die G001
		rcall	Wait						; wieder losgelassen wurde
G201:
		inc		R18							; wenn innerhalb von 2 s kein
		ldi		R16		,	1				; Tastendruck erfolgt, dann wird
		rcall	Wait						; die zuletzt gewhlte Funktion
		cpi		R18		,	200				; bernommen
		breq	G202
		sbic	PINC	,	2				; warten auf Tastendruck
		rjmp	G201
		rjmp	G203
G202:
		pop		R19
		pop		R18
		pop		R16
		sei
		rjmp	Loop
G203:
		cpi		R19		,	5
		breq	G204
		rjmp	G205
G204:
		ldi		R19		,	0
G205:
		inc		R19
		cpi		R19		,	1
		breq	G211
		cpi		R19		,	2
		breq	G212
		cpi		R19		,	3
		breq	G213
		cpi		R19		,	4
		breq	G214
G211:
		rjmp	G221
G212:
		rjmp	G222
G213:
		rjmp	G223
G214:
		rjmp	G224
G221:										; "BAR GRAPH  NONE "
		ldi		ZL		,	LOW(Text23_BGNone*2)
		ldi		ZH		,	HIGH(Text23_BGNone*2)
		rcall	H201
		ldi		R16		,	0
		mov		R27		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G201
G222:										; "BAR GRAPH TYPE 1"
		ldi		ZL		,	LOW(Text24_BGType1*2)
		ldi		ZH		,	HIGH(Text24_BGType1*2)
		rcall	H201
		ldi		R16		,	1
		mov		R27		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G201
G223:										; "BAR GRAPH TYPE 2"
		ldi		ZL		,	LOW(Text25_BGType2*2)
		ldi		ZH		,	HIGH(Text25_BGType2*2)
		rcall	H201
		ldi		R16		,	2
		mov		R27		,	R16
		ldi		R16		,	50
		rcall	wait
		rjmp	G201
G224:										; "CALIBRATION MODE"
		ldi		ZL		,	LOW(Text26_CalMode*2)
		ldi		ZH		,	HIGH(Text26_CalMode*2)
		rcall	H201
		ldi		R16		,	3
		mov		R27		,	R16
		ldi		R16		,	50
		rcall	wait
		ldi		R16		,	4
		mov		R10		,	R16
		rjmp	G201

;-----------------------------------------------------------------------------------------
; Subroutine zur Anzeige von Texten in LCD_line1

H101:
		rcall	LCD_line1
H102:
		lpm		R16		,	Z+
		cpi		R16		,	255
		breq	H103
		rcall	LCD_data
		rjmp	H102
H103:
		ret

;-----------------------------------------------------------------------------------------
; Subroutine zur Anzeige von Texten in LCD_line2

H201:
		rcall	LCD_line2
H202:
		lpm		R16		,	Z+
		cpi		R16		,	255
		breq	H203
		rcall	LCD_data
		rjmp	H202
H203:
		ret

;-----------------------------------------------------------------------------------------
; Zeichen frs CGRam des LCD, Bargraph alle Zeichen leer

Text00_Symbole0:
		.db	0,0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0,0
		.db	0,0,0,0,0,0,0,0,255,0

;-----------------------------------------------------------------------------------------
; Zeichen frs CGRam des LCD, Bargraph mit Leerstellen zwischen den Zeichen
Text01_Symbole1:
		.db	0,0,0,0,0,0,0,0
		.db	0,0,0,16,16,16,0,0
		.db	0,0,0,24,24,24,0,0
		.db	0,0,0,28,28,28,0,0
		.db	0,0,0,30,30,30,0,0
		.db	0,0,0,31,31,31,0,0,255,0

;-----------------------------------------------------------------------------------------
; Zeichen frs CGRam des LCD, Bargraph mit Leerstallen von Bar zu Bar

Text02_Symbole2:
		.db	0,0,0,0,0,0,0,0
		.db	0,0,0,16,16,16,0,0
		.db	0,0,0,16,16,16,0,0
		.db	0,0,0,20,20,20,0,0
		.db	0,0,0,20,20,20,0,0
		.db	0,0,0,21,21,21,0,0,255,0

;-----------------------------------------------------------------------------------------
; Continuity Test

Text04_ContTest:

		.db "CONTINUITY TEST ",255,0

;-----------------------------------------------------------------------------------------
; Low Battery

Text05_LowBat:

		.db "  LOW BATTERY   ",255,0

;-----------------------------------------------------------------------------------------
; Empty Line

Text06_LineEmpty:

		.db "                ",255,0

;-----------------------------------------------------------------------------------------
; Overflow

Text07_Overflow:

		.db "    OVERFLOW    ",255,0

;-----------------------------------------------------------------------------------------
; SelRange

Text09_SelRange:

		.db "Select the Range",255,0

;-----------------------------------------------------------------------------------------
; DC Auto

Text10_DCAuto:

		.db "DC Range: AUTO  ",255,0

;-----------------------------------------------------------------------------------------
; DC 0.25 V

Text11_DC0_25:

		.db "DC Range: 0.25 V",255,0

;-----------------------------------------------------------------------------------------
; DC 2.5 V

Text12_DC2_5:

		.db "DC Range: 2.5 V ",255,0

;-----------------------------------------------------------------------------------------
; DC 25 V

Text13_DC25:

		.db "DC Range: 25 V  ",255,0

;-----------------------------------------------------------------------------------------
; DC 250 V

Text14_DC250:

		.db "DC Range: 250 V ",255,0

;-----------------------------------------------------------------------------------------
; AC Auto

Text15_ACAuto:

		.db "AC Range: AUTO  ",255,0

;-----------------------------------------------------------------------------------------
; AC 0.25 V

Text16_AC0_25:

		.db "AC Range: 0.25 V",255,0

;-----------------------------------------------------------------------------------------
; AC 2.5 V

Text17_AC2_5:

		.db "AC Range: 2.5 V ",255,0

;-----------------------------------------------------------------------------------------
; AC 25 V

Text18_AC25:

		.db "AC Range: 25 V  ",255,0

;-----------------------------------------------------------------------------------------
; AC 250 V

Text19_AC250:

		.db "AC Range: 250 V ",255,0

;-----------------------------------------------------------------------------------------
; DC Auto MAX MIN

Text20_MaxMin:

		.db "DC AUTO MAX MIN ",255,0

;-----------------------------------------------------------------------------------------
; Text21_Calib

Text21_Calib:

		.db " CALIBR. R24=   ",255,0

;-----------------------------------------------------------------------------------------
; LCD_lin1_Text0

Text22_SelMode:

		.db "Select the Mode ",255,0

;-----------------------------------------------------------------------------------------
; Text23_BGNone

Text23_BGNone:

		.db "BAR GRAPH  NONE ",255,0

;-----------------------------------------------------------------------------------------
; Text24_BGType1

Text24_BGType1:

		.db "BAR GRAPH TYPE 1",255,0

;-----------------------------------------------------------------------------------------
; Text25_BGType2

Text25_BGType2:

		.db "BAR GRAPH TYPE 2",255,0

;-----------------------------------------------------------------------------------------
; Text26_CalMode

Text26_CalMode:

		.db "CALIBRATION MODE",255,0

;-----------------------------------------------------------------------------------------


































